import pandas as pd


def clean(covid_df, pop_df):
    """
    This function cleans data frame and returns data frames.
    """
    covid_df = covid_df
    pop_df = pop_df

    # Maps to the correct column names
    pop_df_col_map = {'STATE': 'State_ID', 'COUNTY': 'County_ID', 'POPESTIMATE2019': 'Population'}
    pop_df.columns = [pop_df_col_map.get(x, "No_key") for x in pop_df.columns]

    covid_df_col_map = {'date': 'Date', 'county': 'County', 'state': 'State', 'CTYNAME': 'County',
                        'fips': 'FIPS', 'cases': 'Cases', 'deaths': 'Deaths'}
    covid_df.columns = [covid_df_col_map.get(x, "No_key") for x in covid_df.columns]

    # Combining State FIPS ID and County FIPS ID to 1 column and adding leading zeros
    pop_df['State_ID'] = pop_df['State_ID'].map('{:0>2}'.format)
    pop_df['County_ID'] = pop_df['County_ID'].map('{:0>3}'.format)
    pop_df['FIPS'] = pop_df['State_ID'] + pop_df['County_ID']
    pop_df = pop_df.drop(['State_ID', 'County_ID'], axis=1)
    covid_df['FIPS'] = covid_df['FIPS'].fillna(0).astype(int)
    covid_df['FIPS'] = covid_df['FIPS'].map('{:0>5}'.format)

    # standardizing the date column
    covid_df['Date'] = pd.to_datetime(covid_df['Date'], format='%Y-%m-%d')

    # sorting covid_df by Date for better understanding
    covid_df = covid_df.sort_values(by='Date', ascending=True, ignore_index=True)

    return (covid_df, pop_df)


import pandas as pd

def final_merge(covid_df, pop_df):
    covid_df = covid_df
    pop_df = pop_df
    final = pd.merge(covid_df, pop_df, on=['FIPS'], how='left')
    final['Population'] = final['Population'].fillna(0).astype(int)
    final.columns = ['Date', 'County', 'State', 'FIPS', 'Daily Cases', 'Daily Deaths', 'Population']
    final = final.sort_values(by=['Date', 'State', 'County'], ascending=[True, True, True])
    return final


config = {
    "covid_df": "https://raw.githubusercontent.com/nytimes/covid-19-data/master/us-counties.csv",
    "pop_df":"https://www2.census.gov/programs-surveys/popest/datasets/2010-2019/counties/totals/co-est2019-alldata.csv"
}



import os


def output_path(path):
    """
    This function takes input path and checks if file exist.
    """
    a = True
    while a == True:
        path = input(path)
        if os.path.isdir(path):
            a = False
            return path
        else:
            print('Please enter a valid input path')



import pandas as pd


# This class has the necessary functions to generate summary statistics
class SummaryStats:
    """
    This class gives basic summary statistics
    """

    def freq(df):
        """
        This function gives frequency table of the dataframe and prints the frequency table.
        """
        df = df.select_dtypes(include=['object'])
        col = df.columns
        for i in col:
            print(pd.crosstab(index=df[i], columns='count').sort_values(by='count', ascending=False), '\n')

    def summarize(df, path):
        """
        This function give final cumulative summary table of the dataframe and outputs to a specified path.
        """
        df['Cumulative Cases'] = df.groupby(by=['FIPS'])['Daily Cases'].transform(lambda x: x.cumsum())
        df['Cumulative Deaths'] = df.groupby(by=['FIPS'])['Daily Deaths'].transform(lambda x: x.cumsum())
        df = df[['Date', 'County', 'State', 'FIPS', 'Population', 'Daily Cases', 'Daily Deaths',
                 'Cumulative Cases', 'Cumulative Deaths']]
        df.to_csv(path + '\summary.csv', index=False)



#importing required libraries
from Tiger_Assessment import config, data_clean, IO_path, merge, summary_stats
import pandas as pd

# Setting input and output path variables
# input_pathnames
covid = config.get('covid_df')
pop = config.get('pop_df')
output_pathname = IO_path.output_path('Enter output path folder:')

# Read Data Files
covid_df = pd.read_csv(covid)
pop_df = pd.read_csv(pop, encoding='latin-1',
                     usecols = ['POPESTIMATE2019', 'STATE', 'COUNTY'])

# Cleaning Data Files
(covid_df, pop_df) = data_clean.clean(covid_df, pop_df)


# Merging the data files
final = merge.final_merge(covid_df, pop_df)

# Generating summary file
summary_stats.SummaryStats.summarize(final, output_pathname)


'''
# Covid - 19 Daily Cumulative Statistics

##### The below project is a part of HCSC Machine Learning Engineer position.
As part of HCSC's COVID19 response, the Data Science team needs to prepare daily/weekly updates of nationwide infection counts, organized by county. We use numeric FIPS code https://en.wikipedia.org/wiki/FIPS_county_code rather than
state and county name to serve our results.

For every FIPS code and date, the program generates: population, daily cases, daily deaths, cumulative cases to date, and cumulative death counts to date.

## Citations
The data is supplied by [New York Times](https://www.nytimes.com/interactive/2020/us/coronavirus-us-cases.html).

For details on the data extraction please refer https://github.com/nytimes/covid-19-data

# Program Execution
The goal of the project is to generate a daily/weekly updates of nationwide infection counts, organized by county. Below is the step by step process of executing this program.
The user imports *Tiger_Assessment* library from pip by running the following command(<b>pip install Tiger-Assessment</b>).

The user runs (<b>import Tiger_Assessment</b>). This opens up a GUI in which the user have to provide   **Output Folder Path**

## Data Files

As a part of this project, there are 2 csvs files provided by [New York Times](https://raw.githubusercontent.com/nytimes/covid-19-data/master/us-counties.csv) and [US Censes Data](https://www2.census.gov/programs-surveys/popest/datasets/2010-2019/counties/totals/co-est2019-alldata.csv). The path of the output file directory is given by the user.

## Libraries
Below are the libraries used as a part of this project.

 - pandas
 - numpy
 - os
 - datetime

## Project Files & Folders

 <ul>
   <li><b>Tiger_Assessment</b></li>
   <p>This folder just has the init.py file required to initiate the package and program</p>
   <li><b>config.py</b></li>
   <p>This file initial configuration setting like paths etc.</p>
   <li><b>LICENSE</b></li>
   <p>This is an MIT license</p>
   <li><b>setup.py</b></li>
   <p>This is a setup file required by python to package and distribute the code. This file has all the indetail description and specifications.</p>
   <li><b>my_functions.py</b></li>
   <p>This file has all the classes and functions required for the project</p>
    <li><b>Tiger_Assessment.py</b></li>
   <p>This is the main file of the project. The user runs this file which will generate the summary file in given output path.</p>
</ul>

## Data Dictionary
### `covid`
| Variable |Class  | Description|
|--|--|--|
|date  |date  |Date of collision death (ymd)|
| County| factor | US County Names |
| State| factor | US State Names |
| FIPS| factor | US FIPS code|
|Cases|    integer|Covid Cases reported per day|
|Deaths|   integer|Covid Deaths reported per day|


### `population`
We are extracting only the required columns from the US Censes data.
| Variable |Class  | Description|
|--|--|--|
| STATE| factor | US State FIPS ID|
|County|   factor|    US County FIPS ID|
|POPESTIMATE2019|  integer|   US population estimate|

## Data Cleaning and Preprocessing
Below are the following steps used to clean and preprocess the data.

### 1. Reading the Data
The path to the input files are given in *config.py*. These files are read using pandas for analysis purposes.

### 2. Cleaning the Data Files
*Data_Process* class has all the necessary functions required to clean the data.

Below are the steps used to clean the data file.

 #### Cleaning and Mapping Columns
  <p>I have used a column dictionarys to map the column names correctly which helps in standardizing the column names.</p>

#### Standardizing the Dates
<p>As a best practice, it is always recommended to standardize <i>Dates</i> columns. </p>

#### Sort by Dates
   <p>As a best practice, it is always recommended to sort data by <i>Dates</i> columns. </p>

#### Standarizing FIPS columns.
 1. <p>Population: Concatenating State_ID and County_ID to generate FIPS in population data, so that it can be joined with daily covid data.
   2. Covid: Filling the empty and unknown FIPS IDs with a default value (00000) to standardize the column.</p> .

## Merging the Data Frames
After doing the data preprocessing and clean, we obtain clean files that we can merge. <i>my_func.final_merge</i> takes in two data frames and output one final data frame on which we can do our analysis.

## Generating Summary File
The final step is generate the result. <i>my_func.SummaryStats.summarize</i> generates the summary file as a csv because it is very easy to interpret and do custom analysis on csv.

## Future Edition
### Interactive Plots
We can include interactive plots using pyplot which help the end user analyze the data much more efficiently.
'''


from Tiger_Assessment.config import *
from Tiger_Assessment.data_clean import *
from Tiger_Assessment.data_process import *
from Tiger_Assessment.IO_path import *
from Tiger_Assessment.merge import *
from Tiger_Assessment.summary_stats import *
from Tiger_Assessment.Tiger_Assessment import *



# This python file has all the necessary functions used
import pandas as pd
import numpy as np
import os

class DataProcess:
    """
    This class has all the data cleaning functions required to
    """
    def __init__(self,df):
        return

    def check_null(df):
        """
        This functions takes a Data Frame as input and checks if the variables have any null values.
        It returns a Data Frame with column names and corresponding True/False if it has any nulls.
        """
        colname=df.columns
        na_array = []
        for i in colname:
            # appends column name to temporary array
            na_array.append(str(i))
            # appends True/False after checking if it has any nulls
            na_array.append(df[i].isnull().values.any())
        res = pd.DataFrame(np.reshape(na_array, (-1,2)), columns = ['Column Names', 'Has Nulls'])
        return res


    def capital(df):
        """
        This function takes data frame and capitalises every word of object columns and returns a data frame.
        """
        df_obj = df.select_dtypes(include=['object'])
        for i in df_obj.columns:
            df[i] = df[i].str.title()
        return df


    def remove_nl_tab(df):
        """
        This function takes data frame and removes new lines and tabs and returns a data frame.
        """
        df = df.replace(regex=['\n', '\t'], value='')
        return df


    def trim(df):
        """
        Trims whitespace from begining and end of each value across all non-numeric columns in a dataframe
        """
        df_obj = df.select_dtypes(['object'])
        df[df_obj.columns] = df_obj.apply(lambda x: x.str.strip())
        return df


def clean(covid_df, pop_df):
    """
    This function cleans data frame and returns data frames.
    """
    covid_df = covid_df
    pop_df = pop_df

    # Maps to the correct column names
    pop_df_col_map = {'STATE': 'State_ID', 'COUNTY': 'County_ID', 'POPESTIMATE2019': 'Population'}
    pop_df.columns = [pop_df_col_map.get(x, "No_key") for x in pop_df.columns]

    covid_df_col_map = {'date': 'Date', 'county': 'County', 'state': 'State', 'CTYNAME': 'County',
                        'fips': 'FIPS', 'cases': 'Cases', 'deaths': 'Deaths'}
    covid_df.columns = [covid_df_col_map.get(x, "No_key") for x in covid_df.columns]

    # Combining State FIPS ID and County FIPS ID to 1 column and adding leading zeros
    pop_df['State_ID'] = pop_df['State_ID'].map('{:0>2}'.format)
    pop_df['County_ID'] = pop_df['County_ID'].map('{:0>3}'.format)
    pop_df['FIPS'] = pop_df['State_ID'] + pop_df['County_ID']
    pop_df = pop_df.drop(['State_ID', 'County_ID'], axis=1)
    covid_df['FIPS'] = covid_df['FIPS'].fillna(0).astype(int)
    covid_df['FIPS'] = covid_df['FIPS'].map('{:0>5}'.format)

    # standardizing the date column
    covid_df['Date'] = pd.to_datetime(covid_df['Date'], format='%Y-%m-%d')

    # sorting covid_df by Date for better understanding
    covid_df = covid_df.sort_values(by='Date', ascending=True, ignore_index=True)

    return (covid_df, pop_df)


# This class has the necessary functions to generate summary statistics
class SummaryStats:
    """
    This class gives basic summary statistics
    """

    def freq(df):
        """
        This function gives frequency table of the dataframe and prints the frequency table.
        """
        df = df.select_dtypes(include=['object'])
        col = df.columns
        for i in col:
            print(pd.crosstab(index=df[i], columns='count').sort_values(by='count', ascending=False), '\n')

    def summarize(df, path):
        """
        This function give final cumulative summary table of the dataframe and outputs to a specified path.
        """
        df['Cumulative Cases'] = df.groupby(by=['FIPS'])['Daily Cases'].transform(lambda x: x.cumsum())
        df['Cumulative Deaths'] = df.groupby(by=['FIPS'])['Daily Deaths'].transform(lambda x: x.cumsum())
        df = df[['Date', 'County', 'State', 'FIPS', 'Population', 'Daily Cases', 'Daily Deaths',
                 'Cumulative Cases', 'Cumulative Deaths']]
        df.to_csv(path + '\summary.csv', index=False)


def output_path(path):
    """`
    This function takes input path and checks if file exist.
    """
    a = True
    while a == True:
        path = input(path)
        if os.path.isdir(path):
            a = False
            return path
        else:
            print('Please enter a valid input path')

def final_merge(covid_df, pop_df):
    covid_df = covid_df
    pop_df = pop_df
    final = pd.merge(covid_df, pop_df, on=['FIPS'], how='left')
    final['Population'] = final['Population'].fillna(0).astype(int)
    final.columns = ['Date', 'County', 'State', 'FIPS', 'Daily Cases', 'Daily Deaths', 'Population']
    final = final.sort_values(by=['Date', 'State', 'County'], ascending=[True, True, True])
    return final


# This python file has all the necessary functions used
import pandas as pd
import numpy as np


class DataProcess:
    """
    This class has all the data cleaning functions required to
    """
    def __init__(self,df):
        return

    def check_null(df):
        """
        This functions takes a Data Frame as input and checks if the variables have any null values.
        It returns a Data Frame with column names and corresponding True/False if it has any nulls.
        """
        colname=df.columns
        na_array = []
        for i in colname:
            # appends column name to temporary array
            na_array.append(str(i))
            # appends True/False after checking if it has any nulls
            na_array.append(df[i].isnull().values.any())
        res = pd.DataFrame(np.reshape(na_array, (-1,2)), columns = ['Column Names', 'Has Nulls'])
        return res


    def capital(df):
        """
        This function takes data frame and capitalises every word of object columns and returns a data frame.
        """
        df_obj = df.select_dtypes(include=['object'])
        for i in df_obj.columns:
            df[i] = df[i].str.title()
        return df


    def remove_nl_tab(df):
        """
        This function takes data frame and removes new lines and tabs and returns a data frame.
        """
        df = df.replace(regex=['\n', '\t'], value='')
        return df


    def trim(df):
        """
        Trims whitespace from begining and end of each value across all non-numeric columns in a dataframe
        """
        df_obj = df.select_dtypes(['object'])
        df[df_obj.columns] = df_obj.apply(lambda x: x.str.strip())
        return df


"""Setup for the tiger_assessment package."""

import setuptools


with open('README.md') as f:
    README = f.read()

setuptools.setup(
    author="Sandeep Kiran Gudla",
    author_email="gsandeepkiran@gmail.com",
    name='Tiger_Assessment',
    license='NA',
    description='Tiger_Assessment is a python package for developed as a part of interview process.',
    version='v0.0.17',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/sandeepkirangudla/tiger_assessment',
    packages=setuptools.find_packages(),
    python_requires=">=3.5",
    install_requires=['pandas',
                      'numpy'],
    classifiers=[
        # Trove classifiers
        # (https://pypi.python.org/pypi?%3Aaction=list_classifiers)
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Intended Audience :: Developers',
    ],
)